/**
 * elastic_api_service.js
 * 
 * Service for interacting with Elasticsearch API endpoints
 */
const axios = require('axios');
const logger = require('../../utils/logger');
const { ELASTICSEARCH_CONFIG } = require('../../config/appConfig');

const FILE_NAME = 'elastic_api_service.js';

/**
 * Get Elasticsearch configuration with credentials
 * 
 * @returns {Object} Configuration object with URL and credentials
 */
const getElasticConfig = () => {
  return {
    url: ELASTICSEARCH_CONFIG.apiEndpoint.split('/api/')[0] || process.env.ELASTIC_URL,
    username: ELASTICSEARCH_CONFIG.credentials.split(':')[0] || process.env.ELASTIC_USERNAME,
    password: ELASTICSEARCH_CONFIG.credentials.split(':')[1] || process.env.ELASTIC_PASSWORD,
    apiEndpoint: ELASTICSEARCH_CONFIG.apiEndpoint
  };
};

/**
 * Send a rule to Elasticsearch SIEM
 * 
 * @param {Object} rulePayload - The rule payload to send to Elasticsearch
 * @returns {Promise<Object>} - Object containing success status and response/error information
 */
const sendRuleToSiem = async (rulePayload) => {
  logger.info(`${FILE_NAME}: Sending rule to Elasticsearch SIEM`);
  
  try {
    const elasticConfig = getElasticConfig();
    const apiUrl = elasticConfig.apiEndpoint;
    
    logger.debug(`${FILE_NAME}: Using Elasticsearch API URL: ${apiUrl}`);
    
    // Send the request to Elasticsearch
    const response = await axios({
      method: 'post',
      url: apiUrl,
      headers: {
        'Content-Type': 'application/json',
        'kbn-xsrf': 'true'
      },
      auth: {
        username: elasticConfig.username,
        password: elasticConfig.password
      },
      data: rulePayload
    });
    
    // Process the response
    if (response.status >= 200 && response.status < 300) {
      logger.info(`${FILE_NAME}: Successfully sent rule to SIEM`);
      return {
        success: true,
        status: response.status,
        data: response.data
      };
    } else {
      logger.error(`${FILE_NAME}: Error sending rule to SIEM. Status: ${response.status}, Response: ${JSON.stringify(response.data)}`);
      return {
        success: false,
        status: response.status,
        message: `Failed to add rule to SIEM. Status: ${response.status}`,
        data: response.data
      };
    }
  } catch (error) {
    logger.error(`${FILE_NAME}: API error sending rule to SIEM: ${error.message}`);
    logger.debug(`${FILE_NAME}: API error details: ${error.response ? JSON.stringify(error.response.data) : 'No response data'}`);
    
    const errorMessage = error.response && error.response.data && error.response.data.message
      ? error.response.data.message
      : error.message;
    
    return {
      success: false,
      message: errorMessage,
      error: error
    };
  }
};

/**
 * Make a generic request to an Elasticsearch API endpoint
 * 
 * @param {Object} options - Request options
 * @param {string} options.method - HTTP method (get, post, put, delete)
 * @param {string} options.endpoint - API endpoint (appended to base URL)
 * @param {Object} options.data - Request payload
 * @param {Object} options.params - URL parameters
 * @param {Object} options.headers - Additional headers
 * @returns {Promise<Object>} - Response object
 */
const makeElasticRequest = async (options) => {
  try {
    const elasticConfig = getElasticConfig();
    const baseUrl = elasticConfig.url;
    
    // Build the full URL - use provided endpoint or default API endpoint
    const url = options.endpoint ? 
      `${baseUrl}${options.endpoint.startsWith('/') ? '' : '/'}${options.endpoint}` : 
      elasticConfig.apiEndpoint;
    
    logger.debug(`${FILE_NAME}: Making ${options.method} request to: ${url}`);
    
    // Set up default headers
    const headers = {
      'Content-Type': 'application/json',
      'kbn-xsrf': 'true',
      ...(options.headers || {})
    };
    
    // Make the request
    const response = await axios({
      method: options.method || 'get',
      url: url,
      headers: headers,
      auth: {
        username: elasticConfig.username,
        password: elasticConfig.password
      },
      data: options.data || null,
      params: options.params || null
    });
    
    // Return a standardized response
    return {
      success: response.status >= 200 && response.status < 300,
      status: response.status,
      data: response.data
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in Elasticsearch API request: ${error.message}`);
    
    return {
      success: false,
      message: error.response?.data?.message || error.message,
      status: error.response?.status,
      error: error
    };
  }
};

module.exports = {
  sendRuleToSiem,
  makeElasticRequest,
  getElasticConfig
};